/*******************************/
/*  PIC 18F45K22               */
/*  Clock esterno HS 48MHz     */
/*  MCLRE Disabilitato         */
/*  WatchDog Timer Abilitato   */
/*  WDT Postscaler 1:8192      */
/*******************************/

//Connessione CS per modulo SPI
sbit Mmc_Chip_Select           at LATD3_bit;
sbit Mmc_Chip_Select_Direction at TRISD3_bit;

//Connessioni per I2C Software
sbit Soft_I2C_Scl           at RD4_bit;
sbit Soft_I2C_Sda           at RD5_bit;
sbit Soft_I2C_Scl_Direction at TRISD4_bit;
sbit Soft_I2C_Sda_Direction at TRISD5_bit;

//Porte e direzioni LCD
sbit LCD_RS at RA2_bit;
sbit LCD_EN at RA3_bit;
sbit LCD_D4 at RA4_bit;
sbit LCD_D5 at RA5_bit;
sbit LCD_D6 at RE0_bit;
sbit LCD_D7 at RE1_bit;
sbit LCD_RS_Direction at TRISA2_bit;
sbit LCD_EN_Direction at TRISA3_bit;
sbit LCD_D4_Direction at TRISA4_bit;
sbit LCD_D5_Direction at TRISA5_bit;
sbit LCD_D6_Direction at TRISE0_bit;
sbit LCD_D7_Direction at TRISE1_bit;

#define BackLight PORTE.RE2
#define SET PORTC.RC0
#define CHANGE PORTC.RC1
#define KEY_WAIT 200
#define Write 0xA2
#define Read 0xA3
#define reg_secondi 0x02
#define reg_minuti 0x03
#define reg_ore 0x04
#define reg_giorno 0x05
#define reg_mese 0x07
#define reg_anno 0x08
#define LED PORTD.RD1
#define LED1 PORTD.RD2

int selection = 0;
const char character1[] = {7,15,31,31,21,27,21,31};
//char err_txt[20] = "FAT16 not found";
char filename[14] = "DATALOG1.TXT";
char rtrn [] = "\r\n";
short flag = 0;
short rtcint = 0;
char ch;
char buffer[50];
unsigned short index = 0;
char numero_consentito[] = "+393402659130";
unsigned int adc_value;
char level[7] = "00.00";
long temp;

//Variabili per lettura RTC
char secondi = 0;
char minuti = 0;
char ore = 0;
char giorno = 1;
char giorno_settimana;
char mese = 1;
char anno = 1;
char secolo;
char sampling = 15;
char time [9];
char date [11];
char bufferLcd [3];

//Dichiarazione comandi AT GSM
char atc0[] = "AT";
char atc1[] = "ATE0";                    //Disabilita echo comandi
char atc2[] = "ATH";                     //Rifiuta chiamata
char atc3[] = "AT+CRC=1";                //Abilita modalit report esteso
char atc4[] = "AT+CLIP=1";               //Abilita clipping riconoscimento chiamata
char atc5[] = "AT+CMGF=1";               //Abilita scrittura modalit testo SMS
char atc6[] = "AT+CMGS=+393406642911";   //Invio SMS a numero indicato
char atc7[] = "AT+CMGD=1";               //Elimina messaggio in primo slot memoria sim

//Dichiarazione funzioni
void initSettings();
void initSdCard();
void uartGo();
void gsmConfig ();
void sendAtc (char *s);
void wait ();
void pause ();
void adcResult ();
void fixedLcd ();
void CharADC(char pos_row, char pos_char);
void resetWdt();
void rtcWrite(char address, char reg, char variabile);
void rtcRead();
void rtcConversion();
void rtcDisplay();
char rtcSet(char variabile, char max, char max_value, char min, char address, char reg);
void Character_SD(char pos_row, char pos_char);
void rtcIntWrite ();
void sampleLcd();
void writeSdCard();
void M_Create_New_File();

//Funzione Interrupt
void Interrupt() iv 0x0008 ics ICS_OFF
{
if (INTCON.RBIF)
   {
   if (! PORTB.RB5)
      {
      LED = ! LED;
      flag = 1;
      }

   if (! PORTB.RB4)
      {
      LED1 = ! LED1;
      rtcint = 1;
      }

   INTCON.RBIF = 0;
   }
}

void main()
{
initSettings();
fixedLcd();
initSdCard();
Lcd_Out (4, 7, "--.--");
gsmConfig();
rtcIntWrite();

while (1)
      {
      resetWdt();
      adcResult();
      uartGo();
      rtcRead();
      rtcConversion();
      rtcDisplay();
      writeSdCard();
      sampleLcd();
      if (! SET)
         {
         Delay_ms (KEY_WAIT);
         selection++;
         if (selection >= 7) selection = 0;
         }//Fine if

         switch (selection)
                {
                case 0:
                Lcd_Out (1, 11, "      ");
                Lcd_Out (2, 9, "        ");
                break;

                case 1:
                rtcSet(giorno, 32, 31, 1, Write, reg_giorno);
                Lcd_Out (1, 11, "<--SET");
                break;

                case 2:
                rtcSet (mese, 13, 12, 1, Write, reg_mese);
                Lcd_Out (1, 11, "<--SET");
                break;

                case 3:
                rtcSet (anno, 98, 97, 0, Write, reg_anno);
                Lcd_Out (1, 11, "<--SET");
                break;

                case 4:
                rtcSet (ore, 24, 23, 0, Write, reg_ore);
                Lcd_Out (2, 9, "<----SET");
                Lcd_Out (1, 11, "      ");
                break;

                case 5:
                rtcSet (minuti, 60, 59, 0, Write, reg_minuti);
                Lcd_Out (2, 9, "<----SET");
                break;

                case 6:
                rtcSet (secondi, 60, 59, 0, Write, reg_secondi);
                Lcd_Out (2, 9, "<----SET");
                break;
                }
      }//Fine while
}//Fine main

//Funzione reset WatchDog Timer
void resetWdt ()
{
asm CLRWDT;
}

//Funzione lettura e conversione ADC Sonda
void adcResult ()
{
adc_value = ADC_Read (0);
Delay_ms (300);
temp = (long) adc_value * 5000;
temp = temp / 1023;

if (temp < 985) temp = 985;
else if (temp > 5000) temp = 5000;

/******** Conversione lettura ADC 0-5V in valore livello 0-10m ********/
/*   Costante di conversione  k = (Liv_max - Liv_min)/(Vmax - Vmin)   */
/*   Livello = [(Vmax - Vmin) * k] + Liv_min                          */
/**********************************************************************/

temp = (temp - 985) * 2.5;  //Conversione in range "livello/metri" da visualizzare

level [0] = temp / 10000 + 48;
level [1] = (temp/1000)%10+48;
level [3] = (temp/100)%10+48;
level [4] = (temp/10)%10+48;
//Lcd_Out (1, 1, " Livello  Vasca ");
Lcd_Out (4, -3, "Level--->");
//Lcd_Chr (2, 9, 246);
Lcd_Chr (4, 12, 'm');
Lcd_Out (4, 7, level);
}

//Funzione Riconoscimento numero e risposta modem
void uartGo ()
{
if (flag)
   {
   while (1)
       {
       while (! UART1_Data_Ready()) ; //Attendo dati su seriale
       if (UART1_Read() == '"') break;
       }

   //Ora i caratteri seguenti, fino alle virgolette, identificano il numero
   index = 0;

   while (1)
       {
       while (! UART1_Data_Ready()) ; // Attendo caratteri in ingresso
       ch = UART1_Read();

       //se sono arrivato alle virgolette, fine del numero
       if (ch == '"') break;

       //salvo la cifra
       buffer[index] = ch;
       index ++ ;
       }
   buffer[index] = '\0';

   //in "buffer" ho solo il numero interessante
   // attendo che il modem finisca di inviare i dati e li scarto
   Delay_ms(200);
   while (UART1_Data_Ready()) UART1_Read();

   //Controllo se il numero  consentito
   if (strcmp (buffer, numero_consentito) == 0)
      {
      //Se il numero  autorizzato, rispondo
      sendAtc (atc2);
      pause ();
      sendAtc (atc6);
      pause ();
      UART1_Write_Text ("Livello vasca ");
      pause ();
      UART1_Write_Text (level);
      pause ();
      UART1_Write (0x1A);
      }

   flag = 0; //Resetto il flag
   }
}

//Funzione lettura data e ora da Real Time Clock
void rtcRead ()
{
Soft_I2C_Start();          //Avvio comunicazione I2C
Soft_I2C_Write (Write);    //Invio l'indirizzo allo slave per Write address A2h
Soft_I2C_Write (2);        //Mi posiziono al registro 02h
Soft_I2C_Stop();           //Mando una condizione di STOP
Soft_I2C_Start();          //seguita da START
Soft_I2C_Write (Read);     //Invio l'indirizzo allo slave per Read address A3h

secondi = Soft_I2C_Read (1);             //Leggo i secondi
minuti = Soft_I2C_Read (1);              //Leggo i minuti
ore = Soft_I2C_Read (1);                 //Leggo le ore
giorno = Soft_I2C_Read (1);              //Leggo il giorno
giorno_settimana = Soft_I2C_Read (1);    //Leggo il giorno della settimana
mese = Soft_I2C_Read (1);                //Leggo il mese
anno = Soft_I2C_Read (1);                //Leggo l'anno
Soft_I2C_Stop();                         //Fermo comunicazione I2C
}

//Funzione conversione Data e Ora in decimale
void rtcConversion ()
{
secondi = Bcd2Dec (secondi & 0x7F);
minuti = Bcd2Dec (minuti & 0x7F);
ore = Bcd2Dec (ore & 0x3F);
giorno = Bcd2Dec (giorno & 0x3F);
mese = Bcd2Dec (mese & 0x1F);
anno = Bcd2Dec (anno & 0xFF);
giorno_settimana = (giorno_settimana & 0x07);
secolo = (mese & 0x80);
}

//Funzione visualizzazione ora e data su LCD
void rtcDisplay ()
{
time [0] = (ore/10)+48;
time [1] = (ore%10)+48;
time [2] = ':';
time [3] = (minuti/10)+48;
time [4] = (minuti%10)+48;
time [5] = ':';
time [6] = (secondi/10)+48;
time [7] = (secondi%10)+48;
time [8] = '\0';
Lcd_Out (2, 1, time); //1 12 //1 4

date [0] = (giorno/10)+48;
date [1] = (giorno%10)+48;
date [2] = '/';
date [3] = (mese/10)+48;
date [4] = (mese%10)+48;
date [5] = '/';
date [6] = '2';
date [7] = (secolo+48);
date [8] = (anno/10)+48;
date [9] = (anno%10)+48;
date [10] = '\0';
Lcd_Out (1, 1, date);   //1 1  //2 4
}

//Funzione scrittura dati nella SD Card
void writeSdCard()
{
if (rtcint)
   {
   Mmc_Fat_Assign("DATALOG1.TXT", 0xA0); //Open file
   Mmc_Fat_Append();                     //Append the following data to file...
   Mmc_Fat_Write (date, 11);   //Scrivo la data nel file .txt
   Mmc_Fat_Write (time, 9);    //Scrivo l'ora nel file .txt
   Mmc_Fat_Write (level, 7);   //Scrivo il livello nel file .txt
   Mmc_Fat_Write(rtrn,2);      //Do "l'invio" per passare a nuova riga nel file .txt

   //Resetto il timer di Interrupt dell'RTC per prossimo impulso
   Soft_I2C_Start();
   Soft_I2C_Write (Write);
   Soft_I2C_Write (0x01);
   Soft_I2C_Write (0x01);
   Soft_I2C_Stop();
   
   rtcint = 0;     //Resetto il flag
   }
}

void sampleLcd()
{
Lcd_Out (3, -3, "Sample ");
bufferLcd [0] = (sampling/10)+48;
bufferLcd [1] = (sampling%10)+48;
bufferLcd [2] = '\0';
Lcd_Out(3, 4, bufferLcd);
}

//Funzione per impostazione valori data ed ora del Real Time Clock
char rtcSet (char variabile, char max, char max_value, char min, char address, char reg)
{
if (! CHANGE)
   {
   Delay_ms (KEY_WAIT);
   if (variabile < max)
      {
      variabile++;
      if (variabile > max_value) variabile = min;
      }
   }

rtcWrite (address, reg, variabile);

return variabile;
}

//Funzione scrittura valori RTC
void rtcWrite (char address, char reg, char variabile)
{
Soft_I2C_Start();
Soft_I2C_Write (address);
Soft_I2C_Write (reg);
Soft_I2C_Write (Dec2Bcd (variabile));
Soft_I2C_Stop();
}

//Funzione inizializzazione ed impostazione Interrupt da Real Time Clock
void rtcIntWrite ()
{
Soft_I2C_Start();
Soft_I2C_Write (Write);
Soft_I2C_Write (0x01);    //reg1 0x01
Soft_I2C_Write (0x01);    //reg1_value 0x01
Soft_I2C_Start();
Soft_I2C_Write (Write);
Soft_I2C_Write (0x0F);    //reg2 0x0F
Soft_I2C_Write (0x0F);    //variabile 0x0F
Soft_I2C_Start();
Soft_I2C_Write (Write);
Soft_I2C_Write (0x0E);    //reg3 0x0E
Soft_I2C_Write (0x83);    //reg3_value 0x083
Soft_I2C_Stop();
}

//Funzione per invio comandi AT al modem GSM
void sendAtc(char *s)
{
   while(*s) {
      UART1_Write(*s++);  //Invio stringa comando
   }
   UART1_Write('\r');     //Invio CR esecuzione comando
}

//Funzione configurazione modem all'accensione
void gsmConfig ()
{
sendAtc (atc0);
pause ();
sendAtc (atc1);
pause ();
sendAtc (atc3);
pause ();
sendAtc (atc4);
pause ();
sendAtc (atc5);
}

//Funzione creazione file .txt su SD Card
void M_Create_New_File()
{
Mmc_Fat_Assign ("DATALOG1.TXT", 0xA0);
}

//Funzione visualizzazione di avvio su LCD
void fixedLcd ()
{
Lcd_Out (1, 1, "..initializing..");
Lcd_Out (2, 1, "Please Wait  15s");
pause ();
//Lcd_Out (1, 1, "..initializing..");
Lcd_Out (2, 14, "14s");
pause ();
//Lcd_Out (1, 1, "..initializing..");
Lcd_Out (2, 14, "13s");
pause ();
//Lcd_Out (1, 1, "..initializing..");
Lcd_Out (2, 14, "12s");
pause ();
//Lcd_Out (1, 1, "..initializing..");
Lcd_Out (2, 14, "11s");
pause ();
//Lcd_Out (1, 1, "..initializing..");
Lcd_Out (2, 14, "10s");
pause ();
//Lcd_Out (1, 1, "..initializing..");
Lcd_Out (2, 14, "09s");
pause ();
//Lcd_Out (1, 1, "..initializing..");
Lcd_Out (2, 1, "08s");
pause ();
//Lcd_Out (1, 1, "..initializing..");
Lcd_Out (2, 14, "07s");
pause ();
//Lcd_Out (1, 1, "..initializing..");
Lcd_Out (2, 14, "06s");
pause ();
//Lcd_Out (1, 1, "..initializing..");
Lcd_Out (2, 14, "05s");
pause ();
//Lcd_Out (1, 1, "..initializing..");
Lcd_Out (2, 14, "04s");
pause ();
//Lcd_Out (1, 1, "..initializing..");
Lcd_Out (2, 14, "03s");
pause ();
//Lcd_Out (1, 1, "..initializing..");
Lcd_Out (2, 14, "02s");
pause ();
//Lcd_Out (1, 1, "..initializing..");
Lcd_Out (2, 14, "01s");
pause ();
//Lcd_Out (1, 1, ">>>>>SYSTEM<<<<<");
//Lcd_Out (2, 1, ">>>>>>>ON<<<<<<<");
Lcd_Out (1, 1, ">>>>>>>><<<<<<<<");
Lcd_Out (2, 1, ">    SYSTEM    <");
Lcd_Out (3, -3, ">      ON     <");
Lcd_Out (4, -3, ">>>>>>>><<<<<<<<");
wait ();
Lcd_Cmd (_LCD_CLEAR);
}

//Funzione ritardo
void wait ()
{
Delay_ms (2000);
}

//Funzione pausa
void pause ()
{
Delay_ms (1000);
}

//Funzione carattere custom per segnalazione errore SD Card
void Character_SD(char pos_row, char pos_char) 
{
char i;
Lcd_Cmd(64);
for (i = 0; i<=7; i++) Lcd_Chr_CP(character1[i]);
Lcd_Cmd(_LCD_RETURN_HOME);
Lcd_Chr(pos_row, pos_char, 0);
}

//Funzione inizializzazione SPI e controllo SD Card
void initSdCard()
{
//Inizializza modulo SPI
SPI1_Init_Advanced(_SPI_MASTER_OSC_DIV64, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_LOW, _SPI_LOW_2_HIGH);

//Controllo Presenza Partizione FAT16 per SD Card
if (Mmc_Fat_Init() == 0)
   {
   //Reinizializza modulo SPI ad alta velocit
   SPI1_Init_Advanced(_SPI_MASTER_OSC_DIV4, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_LOW, _SPI_LOW_2_HIGH);
   //Crea File TXT nella root della SD Card
   M_Create_New_File();
   }

//Se la partizione non  presente
else
    {
    //Lcd_Out (3, -3, err_txt);
    Character_SD (3, 12);
    }
}

//Funzione configurazione iniziale PIC
void initSettings ()
{
ANSELA = 0b00000001;   //PortA tutti pin digitali - RA0 ingresso analogico ADC
ANSELB = 0b00000000;   //PortB tutti pin digitali
ANSELC = 0b00000000;   //PortC tutti pin digitali
ANSELD = 0b00000000;   //PortD tutti pin digitali
ANSELE = 0b00000000;   //PortE tutti pin digitali

PORTA = 0;             //PortA tutti ingressi
TRISA = 0b11000011;

PORTB = 0;
TRISB = 0b11111111;

PORTC = 0;
TRISC = 0b10010111;

PORTD = 0;
TRISD = 0b11110001;

PORTE = 0;
TRISE = 0b00001000;

//Abilito LCD
Lcd_Init();
Lcd_Cmd (_LCD_CURSOR_OFF);
Lcd_Cmd (_LCD_CLEAR);
BackLight = 1;

//Abilito UART
UART1_Init(9600);
Delay_ms (100);

//Abilito ADC
ADC_Init ();

//Abilito Modulo I2C - Freq. 400kHz
Soft_I2C_Init();

//Interrupt modalit compatibile
RCON.IPEN = 0;

//Abilito Interrupt globale
INTCON.GIE = 1;

//Abilito Interrupt su PortB
INTCON.RBIE = 1;
}